@@ -1,5 +1,5 @@
       subroutine getvofh(bw,kern,wght,vol)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision bw,wght(2),vol,sofw3D
       external sofw3D
@@ -7,7 +7,7 @@
       RETURN
       END
       double precision function sofw3D(bw,kern,wght)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision bw,wght(2)
       integer j1,j2,j3,dlw1,dlw2,dlw3,clw1,clw2,clw3,ih1,ih2,ih3
@@ -47,72 +47,25 @@ C
       sofw3D=sw*sw/sw2
       RETURN
       END
+      
       subroutine sofw3Df(bw,kern,wght,fw)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision bw,wght(2),fw,sofw3D
       external sofw3D
       fw=sofw3D(bw,kern,wght)
       RETURN
       END
-      
-      double precision function sofw3D0(bw,kern,wght)
-      implicit logical(a-z)
-      integer kern
-      double precision bw,wght(2)
-      integer j1,j2,j3,dlw1,dlw2,dlw3,clw1,clw2,clw3,ih1,ih2,ih3
-      double precision sw,h2,lkern,z1,z2,z3,z
-      external lkern
-      h2=bw*bw
-C
-C   first calculate location weights
-C
-      ih3=FLOOR(bw/wght(2))
-      ih2=FLOOR(bw/wght(1))
-      ih1=FLOOR(bw)
-      dlw1=2*ih1+1
-      dlw2=2*ih2+1
-      dlw3=2*ih3+1
-      clw1=(dlw1+1)/2
-      clw2=(dlw2+1)/2
-      clw3=(dlw3+1)/2
-      sw=0.d0
-      DO j3=1,dlw3
-         z3=(clw3-j3)*wght(2)
-         z3=z3*z3
-         ih2=FLOOR(sqrt(h2-z3)/wght(1))
-         DO j2=clw2-ih2,clw2+ih2
-            z2=(clw2-j2)*wght(1)
-            z2=z3+z2*z2
-            ih1=FLOOR(sqrt(h2-z2))
-            DO j1=clw1-ih1,clw1+ih1
-               z1=clw1-j1
-               z=lkern(kern,(z1*z1+z2)/h2)
-               sw=sw+z
-            END DO
-         END DO
-      END DO
-      sofw3D0=sw
-      RETURN
-      END
-      subroutine ni2var(bw,kern,wght,quot)
-      implicit logical(a-z)
-      integer kern
-      double precision bw,wght(2),quot
-      double precision sofw3D0,sofw3D
-      external sofw3D0,sofw3D
-      quot=sofw3D0(bw,kern,wght)/sofw3D(bw,kern,wght)
-      RETURN
-      END
+
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 C
-C   determine sum of location weights for a given geometry a(3) and given 
+C   determine sum of location weights for a given geometry a(3) and given
 C   bandwidth
 C
 CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 C  Algorithmus zur Nullstellenbestimmung einer monotonen Funktion auf(0,\infty)
       subroutine gethani(x,y,kern,value,wght,eps,bw)
-      implicit logical(a-z)
+      implicit none
       integer kern
       double precision x,y,value,wght(2),eps,bw
       double precision fw1,fw2,fw3,z
@@ -149,11 +102,11 @@ C         z=x+(value-fw1)/(fw2-fw1)*(y-x)
           bw=y-(fw2-value)/(fw2-fw1)*(y-x)
       ENDIF
       RETURN
-      END  
+      END
 
       subroutine mcorrlag(res,mask,n1,n2,n3,nv,scorr,lag)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv,lag(3)
       double precision scorr,res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -193,68 +146,9 @@ C  correlation in x
       scorr=z/k
       return
       end
-      subroutine mcorlag1(res,mask,indm,nvox,n1,n2,n3,nv,scorr,lag)
-      implicit logical(a-z)
-      integer n1,n2,n3,nv,lag(3),nvox,indm(nvox)
-      double precision scorr,res(nv,nvox)
-      logical mask(n1,n2,n3)
-      double precision z2,y2,resi,resip1,vrm,vrmp1,zk,zcorr,z
-      integer i1,i2,i3,i4,l1,l2,l3,k,i,j,m,m0
-      zk=nv
-      l1=lag(1)
-      l2=lag(2)
-      l3=lag(3)
-      z=0.d0
-      k=0
-C  correlation in x
-      do j=1,nvox
-         i=indm(j)
-         i3=i/n1/n2+1
-         i=i-(i3-1)*n1*n2
-         if(i.eq.0) THEN
-            i=n1*n2
-            i3=i3-1
-         ENDIF
-         i2=i/n1+1
-         i1=i-(i2-1)*n1
-         if(i1.eq.0) THEN
-            i1=n1
-            i2=i2-1
-         ENDIF
-         if(i3.gt.n3-l3) CYCLE
-         if(i2.gt.n2-l2) CYCLE
-         if(i1.gt.n1-l1) CYCLE
-         if (.not.mask(i1+l1,i2+l2,i3+l3)) CYCLE
-         z2=0.d0
-         y2=0.d0
-         zcorr=0.d0
-C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
-         i=i1+l1+(i2+l2-1)*n1+(i3+l3-1)*n1*n2
-         DO m=1,nvox
-            m0=m
-            if(i.eq.indm(m)) EXIT
-         END DO
-         do i4=1,nv
-            resi=res(i4,j)
-            resip1=res(i4,m0)
-            z2=z2+resi*resi
-            y2=y2+resip1*resip1
-            zcorr=zcorr+resi*resip1
-         enddo
-         vrm=z2/zk
-         vrmp1=y2/zk
-         vrm=vrm*vrmp1
-         if(vrm.gt.1e-10) THEN
-            z=z+zcorr/zk/sqrt(vrm)
-            k=k+1
-         end if
-      enddo
-      scorr=z/k
-      return
-      end
       subroutine sweepm(res,mask,n1,n2,n3,nv)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv
       double precision res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -277,28 +171,9 @@ C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
       END DO
       return
       end
-      subroutine sweepm0(res,n,nv)
-
-      implicit logical(a-z)
-      integer n,nv
-      double precision res(nv,n)
-      integer i,k
-      double precision z
-      Do i=1,n
-         z=0.d0
-         DO k=1,nv
-            z=z+res(k,i)
-         END DO
-         z=z/nv
-         DO k=1,nv
-            res(k,i)=res(k,i)-z
-         END DO
-      END DO
-      return
-      end
       subroutine mean3D(res,n1,n2,n3,nv,mres)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv
       double precision res(nv,n1,n2,n3),mres(n1,n2,n3)
       integer i1,i2,i3,k
@@ -318,7 +193,7 @@ C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
       end
       subroutine mcorr(res,mask,n1,n2,n3,nv,scorr,l1,l2,l3)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv,l1,l2,l3,lag(3)
       double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -330,27 +205,7 @@ C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
             DO i3=1,l3
                lag(3)=i3-1
                call mcorrlag(res,mask,n1,n2,n3,nv,scorr(i1,i2,i3),lag)
-               call rchkusr()  
-            END DO
-         END DO
-      END DO
-      return
-      end
-      subroutine mcorr1(res,mask,indm,nvox,n1,n2,n3,nv,scorr,l1,l2,l3)
-      implicit logical(a-z)
-      integer n1,n2,n3,nv,l1,l2,l3,lag(3),nvox,indm(nvox)
-      double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
-      logical mask(n1,n2,n3)
-      integer i1,i2,i3
-      Do i1=1,l1
-         lag(1)=i1-1
-         DO i2=1,l2
-            lag(2)=i2-1
-            DO i3=1,l3
-               lag(3)=i3-1
-               call mcorlag1(res,mask,indm,nvox,n1,n2,n3,nv,
-     1                       scorr(i1,i2,i3),lag)
-               call rchkusr()  
+               call rchkusr()
             END DO
          END DO
       END DO
@@ -358,7 +213,7 @@ C   get position in indm corresponding to voxel i1+l1,i2+l2,i3+l3
       end
       subroutine imcorrl(res,mask,n1,n2,n3,nv,scorr,lag)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv,lag(3)
       double precision scorr,res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -400,7 +255,7 @@ C  correlation in x
       end
       subroutine imcorr(res,mask,n1,n2,n3,nv,scorr,l1,l2,l3)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv,l1,l2,l3,lag(3)
       double precision scorr(l1,l2,l3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -412,7 +267,7 @@ C  correlation in x
             DO i3=1,l3
                lag(3)=i3-1
                call imcorrl(res,mask,n1,n2,n3,nv,scorr(i1,i2,i3),lag)
-               call rchkusr()  
+               call rchkusr()
             END DO
          END DO
       END DO
@@ -420,7 +275,7 @@ C  correlation in x
       end
       subroutine thcorr(w,n1,n2,n3,scorr,l1,l2,l3)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,l1,l2,l3,lag(3)
       double precision scorr(l1,l2,l3),w(n1,n2,n3)
       integer i1,i2,i3
@@ -441,7 +296,7 @@ C  correlation in x
                lag(3)=i3-1
                call thcorlag(w,n1,n2,n3,zcorr,lag)
                scorr(i1,i2,i3)=zcorr/z
-               call rchkusr()  
+               call rchkusr()
             END DO
          END DO
       END DO
@@ -449,7 +304,7 @@ C  correlation in x
       end
       subroutine thcorlag(w,n1,n2,n3,scorr,lag)
 
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,lag(3)
       double precision scorr,w(n1,n2,n3)
       integer i1,i2,i3,c1,c2,c3,j1,j2,j3,l1,l2,l3
@@ -459,11 +314,11 @@ C  correlation in x
       c3=(n3-1)/2
       z=0.d0
       Do i1=-c1,c1
-         j1=i1+c1+1 
+         j1=i1+c1+1
          l1=lag(1)-i1+c1+1
          if(l1.lt.1.or.l1.gt.n1) CYCLE
          DO i2=-c2,c2
-            j2=i2+c2+1 
+            j2=i2+c2+1
             l2=lag(2)-i2+c2+1
             if(l2.lt.1.or.l2.gt.n2) CYCLE
             DO i3=-c3,c3
@@ -482,7 +337,7 @@ C  correlation in x
 C
 C   compute variance estimates !!! (not the inverse)
 C
-      implicit logical(a-z)
+      implicit none
       integer n1,n2,n3,nv
       double precision resscale,var(n1,n2,n3),res(nv,n1,n2,n3)
       logical mask(n1,n2,n3)
@@ -516,7 +371,7 @@ C
 C   assumes that we search for a connected region in segm==.TRUE.
 C   that contains seed voxel (i1,i2,i3)
 C   result: mask == .TRUE. if voxel is connected to seed
-      implicit logical (a-z)
+      implicit none
       integer n1,n2,n3,i1,i2,i3,ind1(*),ind2(*),ind3(*)
       logical final,checked(*),mask(n1,n2,n3),segm(n1,n2,n3)
       integer j1,j2,j3,k,l1,l2,l3,lind,lind0,n
@@ -593,7 +448,7 @@ C     first find pixel close to (i1,i2) with segm(j1,j2)=0
                      END IF
                   END DO
                END DO
-            END DO 
+            END DO
          END DO
          if(lind.eq.lind0) THEN
             final=.TRUE.
@@ -603,28 +458,3 @@ C     first find pixel close to (i1,i2) with segm(j1,j2)=0
       END DO
       RETURN
       END
-      subroutine countsgm(s,n1,n2,n3,anz)
-      implicit logical (a-z)
-      integer n1,n2,n3,anz(7)
-      logical s(n1,n2,n3)
-      integer i1,i2,i3,m
-      DO i1=2,n1-1
-         DO i2=2,n2-1
-            DO i3=2,n3-1
-               if(.not.s(i1,i2,i3)) CYCLE
-               m=1
-C  test neighbors
-               if(s(i1-1,i2,i3)) m=m+1
-               if(s(i1+1,i2,i3)) m=m+1
-               if(s(i1,i2-1,i3)) m=m+1
-               if(s(i1,i2+1,i3)) m=m+1
-               if(s(i1,i2,i3-1)) m=m+1
-               if(s(i1,i2,i3+1)) m=m+1
-               anz(m)=anz(m)+1
-            END DO
-         END DO
-      END DO
-      RETURN
-      END
-      
-      
